import sys
import numpy as np
from PyQt4.QtGui import QIntValidator, QDoubleValidator, QApplication, QSizePolicy
# from PyMca5.PyMcaIO import specfilewrapper as specfile
from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import widget
import orangecanvas.resources as resources
import sys,os

class OWfunctions1D(widget.OWWidget):
    name = "functions1D"
    id = "orange.widgets.datafunctions1D"
    description = "Application to compute..."
    icon = "icons/functions1D.png"
    author = "create_widget.py"
    maintainer_email = "srio@esrf.eu"
    priority = 10
    category = ""
    keywords = ["oasysaddontemplate", "functions1D"]
    outputs = [{"name": "oasysaddontemplate-data",
                "type": np.ndarray,
                "doc": "transfer numpy arrays"},
               # another possible output
               # {"name": "oasysaddontemplate-file",
               #  "type": str,
               #  "doc": "transfer a file"},
                ]

    # widget input (if needed)
    #inputs = [{"name": "Name",
    #           "type": type,
    #           "handler": None,
    #           "doc": ""}]

    want_main_area = False

    FROM = Setting(-100.0)
    TO = Setting(100.0)
    NPOINTS = Setting(500)
    FUNCTION_NAME = Setting(3)
    CUSTOM = Setting("np.sin(x)")
    DUMP_TO_FILE = Setting(0)
    FILE_NAME = Setting("tmp.dat")


    def __init__(self):
        super().__init__()

        box0 = gui.widgetBox(self.controlArea, " ",orientation="horizontal") 
        #widget buttons: compute, set defaults, help
        gui.button(box0, self, "Compute", callback=self.compute)
        gui.button(box0, self, "Defaults", callback=self.defaults)
        gui.button(box0, self, "Help", callback=self.get_doc)
        self.process_showers()
        box = gui.widgetBox(self.controlArea, " ",orientation="vertical") 
        
        
        idx = -1 
        
        #widget index 0 
        idx += 1 
        box1 = gui.widgetBox(box) 
        gui.lineEdit(box1, self, "FROM",
                     label=self.unitLabels()[idx], addSpace=True,
                    valueType=float, validator=QDoubleValidator())
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 1 
        idx += 1 
        box1 = gui.widgetBox(box) 
        gui.lineEdit(box1, self, "TO",
                     label=self.unitLabels()[idx], addSpace=True,
                    valueType=float, validator=QDoubleValidator())
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 2 
        idx += 1 
        box1 = gui.widgetBox(box) 
        gui.lineEdit(box1, self, "NPOINTS",
                     label=self.unitLabels()[idx], addSpace=True,
                    valueType=int, validator=QIntValidator())
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 3 
        idx += 1 
        box1 = gui.widgetBox(box) 
        gui.comboBox(box1, self, "FUNCTION_NAME",
                     label=self.unitLabels()[idx], addSpace=True,
                    items=['sin(x)', 'cos(x)', 'x^2+x+1', 'Custom'],
                    valueType=int, orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 4 
        idx += 1 
        box1 = gui.widgetBox(box) 
        gui.lineEdit(box1, self, "CUSTOM",
                     label=self.unitLabels()[idx], addSpace=True)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 5 
        idx += 1 
        box1 = gui.widgetBox(box) 
        gui.comboBox(box1, self, "DUMP_TO_FILE",
                     label=self.unitLabels()[idx], addSpace=True,
                    items=['Yes', 'No'],
                    valueType=int, orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 6 
        idx += 1 
        box1 = gui.widgetBox(box) 
        gui.lineEdit(box1, self, "FILE_NAME",
                     label=self.unitLabels()[idx], addSpace=True)
        self.show_at(self.unitFlags()[idx], box1) 

        gui.rubber(self.controlArea)

    def unitLabels(self):
         return ['Abscissa from ','Abscissa to','Number of points','Function','Custom expression',      'Dump to file','File name']


    def unitFlags(self):
         return ['True',          'True',       'True',            'True',    'self.FUNCTION_NAME == 3','True',        'self.DUMP_TO_FILE == 0']


    def compute(self):
        dataArray = OWfunctions1D.calculate_external_functions1D(FROM=self.FROM,TO=self.TO,NPOINTS=self.NPOINTS,FUNCTION_NAME=self.FUNCTION_NAME,CUSTOM=self.CUSTOM,DUMP_TO_FILE=self.DUMP_TO_FILE,FILE_NAME=self.FILE_NAME)

        # if fileName == None:
        #     print("No file to send")
        # else:
        #     self.send("oasysaddontemplate-file",fileName)

        self.send("oasysaddontemplate-data",dataArray)


    def defaults(self):
         self.resetSettings()
         self.compute()
         return

    def get_doc(self):
        print("help pressed.")
        home_doc = resources.package_dirname("orangecontrib.oasysaddontemplate") + "/doc_files/"
        filename1 = os.path.join(home_doc,'functions1D'+'.txt')
        print("Opening file %s"%filename1)
        if sys.platform == 'darwin':
            command = "open -a TextEdit "+filename1+" &"
        elif sys.platform == 'linux':
            command = "gedit "+filename1+" &"
        os.system(command)


    #
    # this is the calculation method to be implemented by the user
    # It is defined as static method to get all inputs from the arguments so it
    # can easily moved outside the class
    #
    @staticmethod
    def calculate_external_functions1D(FROM=-100.0,TO=100.0,NPOINTS=500,FUNCTION_NAME=3,CUSTOM="sin(x)",DUMP_TO_FILE=0,FILE_NAME="tmp.dat"):
        print("Inside calculate_external_functions1D. ")
        return(None)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = OWfunctions1D()
    w.show()
    app.exec()
    w.saveSettings()



import numpy
from srxraylib.plot.gol import plot_image, plot
import sys

from comsyl.scripts.CompactAFReader import CompactAFReader




def plot_stack(mystack,what="intensity",title0="X",title1="Y",title2="Z"):

    from silx.gui.plot.StackView import StackViewMainWindow
    from silx.gui import qt


    app = qt.QApplication(sys.argv[1:])

    sv = StackViewMainWindow()
    sv.setColormap("jet", autoscale=True)
    if what == "intensity":
        sv.setStack(numpy.absolute(mystack))
    elif what == "real":
        sv.setStack(numpy.real(mystack))
    elif what == "imaginary":
        sv.setStack(numpy.imag(mystack))
    elif what == "phase":
        sv.setStack(numpy.angle(mystack))
    elif what == "phase_deg":
        sv.setStack(numpy.angle(mystack,deg=True))
    else:
        raise Exception("Undefined label "+what)

    sv.setLabels([title0,title1,title2])
    sv.show()

    app.exec_()

def load_stack(filename):
    # filename = "/users/srio/OASYS_VE/comsyl_srio/calculations/new_u18_2m_1h_s2.5"
    reader = CompactAFReader(filename)

    print("File %s:" % filename)
    print("contains")
    print("%i modes" % reader.number_modes())
    print("on the grid")
    print("x: from %e to %e" % (reader.x_coordinates().min(), reader.x_coordinates().max()))
    print("y: from %e to %e" % (reader.y_coordinates().min(), reader.y_coordinates().max()))
    print("calculated at %f eV" % reader.photon_energy())
    print("with total intensity in (maybe improper) normalization: %e" % reader.total_intensity().real.sum())



    print("Occupation and max abs value of the mode")

    x = reader.x_coordinates()
    y = reader.y_coordinates()

    eigenvalues = numpy.zeros(reader.number_modes())

    mystack = numpy.zeros((reader.number_modes(),y.size,x.size),dtype=complex)

    for i_mode in range(reader.number_modes()):
        eigenvalues[i_mode] = reader.occupation(i_mode)
        mode = reader.mode(i_mode)
        mystack[i_mode,:,:] = mode.T


    return x,y,mystack, eigenvalues

if __name__ == "__main__":

    h,v,mystack, occupation = load_stack("/users/srio/OASYS_VE/comsyl_srio/calculations/new_u18_2m_1h_s2.5")

    plot_stack(mystack,what="intensity", title0="Mode index",
               title1="V from %3.2f to %3.2f um"%(1e3*v.min(),1e3*v.max()),
               title2="H from %3.2f to %3.2f um"%(1e3*h.min(),1e3*h.max()))

    plot(numpy.arange(occupation.size),occupation)


#
# TODO:
#      - Add progress bar


from PyQt5.QtWidgets import QApplication
from PyQt5.QtGui import QPalette, QColor, QFont


from silx.gui.plot import PlotWindow, Plot2D
from silx.gui.plot.StackView import StackViewMainWindow

import numpy

from orangewidget import gui
from orangewidget.settings import Setting
from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget

from orangewidget import widget
from oasys.widgets import congruence
from oasys.widgets import gui as oasysgui

from wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D
from wofryimpl.beamline.beamline import WOBeamline

from comsyl.autocorrelation.CompactAFReader import CompactAFReader

from orangecontrib.wofry.util.wofry_objects import WofryData
from orangecontrib.comsyl.util.light_source import WOLightSourceCOMSYL

from oasys.util.oasys_util import TriggerIn, TriggerOut


class OWModesSelector(WofryWidget):

    name = "ModesSelector"
    id = "orangecontrib.comsyl.widgets.applications.comsyl_modes_viewer"
    description = ""
    icon = "icons/selector.png"
    author = ""
    maintainer_email = "srio@esrf.fr"
    priority = 45
    category = ""
    keywords = ["COMSYL", "coherent modes"]


    inputs = [("COMSYL modes" , CompactAFReader, "setCompactAFReader" ),
              ("Trigger", TriggerOut, "receive_trigger_signal")]

    outputs = [{"name":"WofryData",
                "type":WofryData,
                "doc":"WofryData",
                "id":"WofryData"},
               {"name":"Trigger",
                "type": TriggerIn,
                "doc":"Feedback signal to start a new beam simulation",
                "id":"Trigger"},
               {"name":"COMSYL modes",
                "type":CompactAFReader,
                "doc":"COMSYL modes",
                "id":"COMSYL modes"},]


    NORMALIZATION = Setting(1) # 0=No, 1=With eigenvalues
    TYPE_PRESENTATION = Setting(0) # 0=intensity, 1=real, 2=phase
    INDIVIDUAL_MODES = Setting(False)
    MODE_INDEX = Setting(0)
    REFERENCE_SOURCE = Setting(0)

    # IS_DEVELOP = True
    _input_available = False

    def __init__(self):

        super().__init__(is_automatic=True, show_view_options=True, show_script_tab=True)

        self.runaction = widget.OWAction("Generate Wavefront", self)
        self.runaction.triggered.connect(self.do_plot_and_send_mode)
        self.addAction(self.runaction)

        gui.separator(self.controlArea)
        gui.separator(self.controlArea)

        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Plot and Send mode", callback=self.do_plot_and_send_mode)
        font = QFont(button.font())
        font.setBold(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Blue'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)

        gui.separator(self.controlArea)

        self.controlArea.setFixedWidth(self.CONTROL_AREA_WIDTH)

        tabs_setting = oasysgui.tabWidget(self.controlArea)
        tabs_setting.setFixedHeight(self.TABS_AREA_HEIGHT + 50)
        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)

        self.tab_settings = oasysgui.createTabPage(tabs_setting, "Settings")


        #
        # Settings
        #

        gui.comboBox(self.tab_settings, self, "NORMALIZATION",
                    label="Renormalize modes ", addSpace=False,
                    items=["No (pure eigenvectors)", "Yes (to carry intensity from eigenvalues)"],
                    valueType=int, orientation="horizontal", callback=self.do_plot_and_send_mode)

        gui.comboBox(self.tab_settings, self, "TYPE_PRESENTATION",
                    label="Display coherent mode ", addSpace=False,
                    items=self.list_TYPE_PRESENTATION(),
                    valueType=int, orientation="horizontal", callback=self.do_plot_and_send_mode)


        gui.comboBox(self.tab_settings, self, "INDIVIDUAL_MODES",
                    label="Load all modes in memory ", addSpace=False,
                    items=['No [Fast, Recommended]','Yes [Slow, Memory hungry]',],
                    valueType=int, orientation="horizontal", callback=self.do_plot_and_send_mode)

        gui.comboBox(self.tab_settings, self, "REFERENCE_SOURCE",
                    label="Display reference source ", addSpace=False,
                    items=['No','Yes',],
                    valueType=int, orientation="horizontal", callback=self.do_plot_and_send_mode)


        mode_index_box = oasysgui.widgetBox(self.tab_settings, "", addSpace=True, orientation="horizontal")

        left_box_5 = oasysgui.widgetBox(mode_index_box, "", addSpace=True, orientation="horizontal", )
        oasysgui.lineEdit(left_box_5, self, "MODE_INDEX", "Send mode",
                        labelWidth=200, valueType=int, tooltip = "mode_index",
                        orientation="horizontal", callback=self.do_plot_and_send_mode)

        gui.button(left_box_5, self, "+1", callback=self.increase_mode_index, width=30)
        gui.button(left_box_5, self, "-1", callback=self.decrease_mode_index, width=30)
        gui.button(left_box_5, self,  "0", callback=self.reset_mode_index, width=30)


    def list_TYPE_PRESENTATION(self):
        return ['intensity','modulus','real part','imaginary part','angle [rad]']

    def get_light_source(self):
        return WOLightSourceCOMSYL(name=self.name,
                                   filename=self.af._filename,
                                   mode_index=self.MODE_INDEX,
                                   normalize_with_eigenvalue=self.NORMALIZATION)

    def initializeTabs(self):
        size = len(self.tab)
        indexes = range(0, size)

        for index in indexes:
            self.tabs.removeTab(size-1-index)

        self.tab = []
        self.plot_canvas = []

        self.set_tab_titles()

        for index in range(0, len(self.tab_titles)):
            self.tab.append(gui.createTabPage(self.tabs, self.tab_titles[index]))
            self.plot_canvas.append(None)

        for tab in self.tab:
            tab.setFixedHeight(self.IMAGE_HEIGHT)
            tab.setFixedWidth(self.IMAGE_WIDTH)

    def setCompactAFReader(self, data):
        if not data is None:
            self.af = data
            self._input_available = True
            self.wofry_output.setText(self.af.info(list_modes=False))
            self.main_tabs.setCurrentIndex(1)
            self.initializeTabs()
            self.do_plot_and_send_mode()

    def receive_trigger_signal(self, trigger):

        if trigger and trigger.new_object == True:
            if trigger.has_additional_parameter("variable_name"):
                variable_name = trigger.get_additional_parameter("variable_name").strip()
                variable_display_name = trigger.get_additional_parameter("variable_display_name").strip()
                variable_value = trigger.get_additional_parameter("variable_value")
                variable_um = trigger.get_additional_parameter("variable_um")

                if "," in variable_name:
                    variable_names = variable_name.split(",")

                    for variable_name in variable_names:
                        setattr(self, variable_name.strip(), variable_value)
                else:
                    setattr(self, variable_name, variable_value)

                self.do_plot_and_send_mode()
            else:
                self.increase_mode_index()

    def increase_mode_index(self):
        if self.MODE_INDEX+1 >= self.af.number_of_modes():
            pass
        else:
            self.MODE_INDEX += 1
            self.do_plot_and_send_mode()

    def decrease_mode_index(self):
        if self.MODE_INDEX-1 < 0:
            pass
        else:
            self.MODE_INDEX -= 1
            self.do_plot_and_send_mode()

    def reset_mode_index(self):
        self.MODE_INDEX = 0
        self.do_plot_and_send_mode()

    def _square_modulus(self,array1):
        return (numpy.absolute(array1))**2

    def _intensity_times_eigenvalue(self,array1):
        s = array1.shape
        if len(s) == 3: # stack
            for i in range(s[0]):
                array1[i] *= numpy.sqrt(self.af.eigenvalue(i).real)
        else:
            array1 *= numpy.sqrt(self.af.eigenvalue(self.MODE_INDEX).real)
        return (numpy.absolute(array1))**2


    def plot_data2D(self, data2D, dataX, dataY, plot_canvas_index, title="", xtitle="", ytitle=""):

        xmin = numpy.min(dataX)
        xmax = numpy.max(dataX)
        ymin = numpy.min(dataY)
        ymax = numpy.max(dataY)

        origin = (xmin, ymin)
        scale = (abs((xmax-xmin)/len(dataX)), abs((ymax-ymin)/len(dataY)))

        data_to_plot = data2D.T

        colormap = {"name":"temperature", "normalization":"linear", "autoscale":True, "vmin":0, "vmax":0, "colors":256}

        self.plot_canvas[plot_canvas_index] = Plot2D()

        self.plot_canvas[plot_canvas_index].resetZoom()
        self.plot_canvas[plot_canvas_index].setXAxisAutoScale(True)
        self.plot_canvas[plot_canvas_index].setYAxisAutoScale(True)
        self.plot_canvas[plot_canvas_index].setGraphGrid(False)
        self.plot_canvas[plot_canvas_index].setKeepDataAspectRatio(True)
        self.plot_canvas[plot_canvas_index].yAxisInvertedAction.setVisible(False)

        self.plot_canvas[plot_canvas_index].setXAxisLogarithmic(False)
        self.plot_canvas[plot_canvas_index].setYAxisLogarithmic(False)
        #silx 0.4.0
        self.plot_canvas[plot_canvas_index].getMaskAction().setVisible(False)
        self.plot_canvas[plot_canvas_index].getRoiAction().setVisible(False)
        self.plot_canvas[plot_canvas_index].getColormapAction().setVisible(True)
        self.plot_canvas[plot_canvas_index].setKeepDataAspectRatio(False)

        self.plot_canvas[plot_canvas_index].addImage(numpy.array(data_to_plot),
                                                     legend="zio billy",
                                                     scale=scale,
                                                     origin=origin,
                                                     colormap=colormap,
                                                     replace=True)


        self.plot_canvas[plot_canvas_index].setGraphXLabel(xtitle)
        self.plot_canvas[plot_canvas_index].setGraphYLabel(ytitle)
        self.plot_canvas[plot_canvas_index].setGraphTitle(title)


        self.tab[plot_canvas_index].layout().addWidget(self.plot_canvas[plot_canvas_index])

    def set_tab_titles(self):
        if self.INDIVIDUAL_MODES:
            self.tab_titles = ["SPECTRUM","CUMULATED SPECTRUM","INDIVIDUAL MODES",]
            if self.REFERENCE_SOURCE:
                self.tab_titles += ["REFERENCE SPECTRAL DENSITY","SPECTRAL INTENSITY FROM MODES","REFERENCE ELECRON DENSITY","REFERENCE UNDULATOR WAVEFRONT"]
        else:
            self.tab_titles = ["SPECTRUM","CUMULATED SPECTRUM","MODE INDEX: %d"%self.MODE_INDEX,]
            if self.REFERENCE_SOURCE:
                self.tab_titles += ["REFERENCE SPECTRAL DENSITY","REFERENCE ELECRON DENSITY","REFERENCE UNDULATOR WAVEFRONT"]

    def do_plot_and_send_mode(self):
        self.do_plot_results()
        self.send_mode()

    def do_plot_results(self, progressBarValue=10):  # required by parent widget
        old_tab_index = self.tabs.currentIndex()

        try:
            for i in range(len(self.tab_titles)):
                self.tab[i].layout().removeItem(self.tab[i].layout().itemAt(0))
        except:
            pass

        if self.view_type != 0:

            self.set_tab_titles()
            # self.initialize_tabs()
            self.initializeTabs()

            if self.TYPE_PRESENTATION == 0:
                myprocess = self._square_modulus
                title0 = "Intensity of eigenvalues"
                title1 = "Intensity of eigenvector"
            if self.TYPE_PRESENTATION == 1:
                myprocess = numpy.absolute
                title0 = "Modulus of eigenvalues"
                title1 = "Modulus of eigenvector"
            elif self.TYPE_PRESENTATION == 2:
                myprocess = numpy.real
                title0 = "Real part of eigenvalues"
                title1 = "Real part of eigenvector"
            elif self.TYPE_PRESENTATION == 3:
                myprocess = numpy.imag
                title0 = "Imaginary part of eigenvalues"
                title1 = "Imaginary part of eigenvectos"
            elif self.TYPE_PRESENTATION == 4:
                myprocess = numpy.angle
                title0 = "Angle of eigenvalues [rad]"
                title1 = "Angle of eigenvector [rad]"


            if self._input_available:
                x_values = numpy.arange(self.af.number_modes())
                x_label = "Mode index"
                y_label = "Occupation"

                xx = self.af.x_coordinates()
                yy = self.af.y_coordinates()

                xmin = numpy.min(xx)
                xmax = numpy.max(xx)
                ymin = numpy.min(yy)
                ymax = numpy.max(yy)

            else:
                print("No input data")
                return

            #
            # plot spectrum
            #
            tab_index = 0
            self.plot_canvas[tab_index] = PlotWindow(parent=None,
                                                     backend=None,
                                                     resetzoom=True,
                                                     autoScale=False,
                                                     logScale=True,
                                                     grid=True,
                                                     curveStyle=True,
                                                     colormap=False,
                                                     aspectRatio=False,
                                                     yInverted=False,
                                                     copy=True,
                                                     save=True,
                                                     print_=True,
                                                     control=False,
                                                     position=True,
                                                     roi=False,
                                                     mask=False,
                                                     fit=False)

            self.tab[tab_index].layout().addWidget(self.plot_canvas[tab_index])

            self.plot_canvas[tab_index].setDefaultPlotLines(True)
            self.plot_canvas[tab_index].setXAxisLogarithmic(False)
            self.plot_canvas[tab_index].setYAxisLogarithmic(False)
            self.plot_canvas[tab_index].setGraphXLabel(x_label)
            self.plot_canvas[tab_index].setGraphYLabel(y_label)
            self.plot_canvas[tab_index].addCurve(x_values, numpy.abs(self.af.occupation_array()), title0, symbol='',
                                                 xlabel="X", ylabel="Y", replace=False)  # '+', '^', ','

            self.tab[tab_index].layout().addWidget(self.plot_canvas[tab_index])
            #
            # plot cumulated spectrum
            #
            tab_index += 1
            self.plot_canvas[tab_index] = PlotWindow(parent=None,
                                                     backend=None,
                                                     resetzoom=True,
                                                     autoScale=False,
                                                     logScale=True,
                                                     grid=True,
                                                     curveStyle=True,
                                                     colormap=False,
                                                     aspectRatio=False,
                                                     yInverted=False,
                                                     copy=True,
                                                     save=True,
                                                     print_=True,
                                                     control=False,
                                                     position=True,
                                                     roi=False,
                                                     mask=False,
                                                     fit=False)

            self.tab[tab_index].layout().addWidget(self.plot_canvas[tab_index])

            self.plot_canvas[tab_index].setDefaultPlotLines(True)
            self.plot_canvas[tab_index].setXAxisLogarithmic(False)
            self.plot_canvas[tab_index].setYAxisLogarithmic(False)
            self.plot_canvas[tab_index].setGraphXLabel(x_label)
            self.plot_canvas[tab_index].setGraphYLabel("Cumulated occupation")
            # self.plot_canvas[tab_index].addCurve(x_values, numpy.cumsum(numpy.abs(self.af.occupation_array())), "Cumulated occupation", symbol='', xlabel="X", ylabel="Y", replace=False) #'+', '^', ','
            self.plot_canvas[tab_index].addCurve(x_values, self.af.cumulated_occupation_array(), "Cumulated occupation",
                                                 symbol='', xlabel="X", ylabel="Y", replace=False)  # '+', '^', ','

            self.plot_canvas[tab_index].setGraphYLimits(0.0, 1.0)

            self.tab[tab_index].layout().addWidget(self.plot_canvas[tab_index])
            #
            # plot all modes
            #

            if self.INDIVIDUAL_MODES:
                tab_index += 1
                dim0_calib = (0, 1)
                dim1_calib = (1e6 * yy[0], 1e6 * (yy[1] - yy[0]))
                dim2_calib = (1e6 * xx[0], 1e6 * (xx[1] - xx[0]))

                colormap = {"name": "temperature", "normalization": "linear", "autoscale": True, "vmin": 0, "vmax": 0,
                            "colors": 256}

                self.plot_canvas[tab_index] = StackViewMainWindow()
                self.plot_canvas[tab_index].setGraphTitle(title1)
                self.plot_canvas[tab_index].setLabels(["Mode number",
                                                       "Y index from %4.2f to %4.2f um" % (1e6 * ymin, 1e6 * ymax),
                                                       "X index from %4.2f to %4.2f um" % (1e6 * xmin, 1e6 * xmax),
                                                       ])
                self.plot_canvas[tab_index].setColormap(colormap=colormap)

                self.plot_canvas[tab_index].setStack(myprocess(numpy.swapaxes(self.af.modes(), 2, 1)),
                                                     calibrations=[dim0_calib, dim1_calib, dim2_calib])

                self.tab[tab_index].layout().addWidget(self.plot_canvas[tab_index])
            else:
                tab_index += 1
                wf = self.af.get_wavefront(self.MODE_INDEX, normalize_with_eigenvalue=self.NORMALIZATION)
                image = myprocess(wf.get_complex_amplitude())

                self.plot_data2D(image,
                                 1e6 * self.af.x_coordinates(),
                                 1e6 * self.af.y_coordinates(),
                                 tab_index,
                                 title="%s; Mode %d" % (self.list_TYPE_PRESENTATION()[self.TYPE_PRESENTATION], self.MODE_INDEX),
                                 xtitle="X [um] (%d pixels)" % (image.shape[0]),
                                 ytitle="Y [um] (%d pixels)" % (image.shape[1]))

            #
            # plot spectral density
            #
            if self.REFERENCE_SOURCE:
                tab_index += 1
                image = myprocess((self.af.spectral_density()))
                # self.do_plot_image_in_tab(image,tab_index,title="Spectral Density (Intensity)")
                self.plot_data2D(image,
                                 1e6 * self.af.x_coordinates(),
                                 1e6 * self.af.y_coordinates(),
                                 tab_index,
                                 title="Spectral Density (Intensity)",
                                 xtitle="X [um] (%d pixels)" % (image.shape[0]),
                                 ytitle="Y [um] (%d pixels)" % (image.shape[1]))

            #
            # plot spectral density from modes
            #
            if self.REFERENCE_SOURCE:
                if self.INDIVIDUAL_MODES:
                    tab_index += 1
                    image = myprocess((self.af.intensity_from_modes()))
                    self.plot_data2D(image,
                                     1e6 * self.af.x_coordinates(),
                                     1e6 * self.af.y_coordinates(),
                                     tab_index,
                                     title="Spectral Density (Intensity)",
                                     xtitle="X [um] (%d pixels)" % (image.shape[0]),
                                     ytitle="Y [um] (%d pixels)" % (image.shape[1]))

            #
            # plot reference electron density
            #
            if self.REFERENCE_SOURCE:
                tab_index += 1
                image = numpy.abs(self.af.reference_electron_density()) ** 2  # TODO: Correct? it is complex...
                self.plot_data2D(image,
                                 1e6 * self.af.x_coordinates(),
                                 1e6 * self.af.y_coordinates(),
                                 tab_index,
                                 title="Reference electron density",
                                 xtitle="X [um] (%d pixels)" % (image.shape[0]),
                                 ytitle="Y [um] (%d pixels)" % (image.shape[1]))

            #
            # plot reference undulator radiation
            #
            if self.REFERENCE_SOURCE:
                tab_index += 1
                image = self.af.reference_undulator_radiation()[0, :, :, 0]  # TODO: Correct? is polarized?

                self.plot_data2D(image,
                                 1e6 * self.af.x_coordinates(),
                                 1e6 * self.af.y_coordinates(),
                                 tab_index,
                                 title="Reference undulator radiation",
                                 xtitle="X [um] (%d pixels)" % (image.shape[0]),
                                 ytitle="Y [um] (%d pixels)" % (image.shape[1]))

            try:
                self.tabs.setCurrentIndex(old_tab_index)
            except:
                pass


    def get_doc(self):
        pass

    def send_mode(self):

        wf = GenericWavefront2D.initialize_wavefront_from_arrays(
                self.af.x_coordinates(),self.af.y_coordinates(), self.af.mode(self.MODE_INDEX)  )
        wf.set_photon_energy(self.af.photon_energy())
        ampl = wf.get_complex_amplitude()

        if self.TYPE_PRESENTATION == 5:
            eigen = self.af.eigenvalues_old()
            wf.set_complex_amplitude(ampl * numpy.sqrt(eigen[self.MODE_INDEX]))
        else:
            wf.set_complex_amplitude(ampl)

        beamline = WOBeamline(light_source=self.get_light_source())
        print(">>> sending mode: ", int(self.MODE_INDEX))
        self.send("WofryData", WofryData(
            wavefront=wf,
            beamline=beamline))

        # script
        self.wofry_python_script.set_code(beamline.to_python_code())


if __name__ == '__main__':

    filename = "/scisoft/users/srio/COMSYL-SLURM/comsyl/comsyl-submit/calculations/id18_ebs_u18_2500mm_s6.0.npz"
    af = CompactAFReader.initialize_from_file(filename)



    app = QApplication([])
    ow = OWModesSelector()

    ow.setCompactAFReader(af)

    ow.show()
    app.exec_()
    ow.saveSettings()





###############################################################
#
# MESSAGING
#
###############################################################

from PyQt5.QtWidgets import QMessageBox

def showConfirmMessage(message, informative_text, parent=None):
    msgBox = QMessageBox()
    if not parent is None: msgBox.setParent(parent)
    msgBox.setIcon(QMessageBox.Question)
    msgBox.setText(message)
    msgBox.setInformativeText(informative_text)
    msgBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
    msgBox.setDefaultButton(QMessageBox.No)

    return msgBox.exec_() == QMessageBox.Yes

def showWarningMessage(message, parent=None):
    msgBox = QMessageBox()
    if not parent is None: msgBox.setParent(parent)
    msgBox.setIcon(QMessageBox.Warning)
    msgBox.setText(message)
    msgBox.setStandardButtons(QMessageBox.Ok)
    msgBox.exec_()

def showCriticalMessage(message, parent=None):
    msgBox = QMessageBox()
    if not parent is None: msgBox.setParent(parent)
    msgBox.setIcon(QMessageBox.Critical)
    msgBox.setText(message)
    msgBox.setStandardButtons(QMessageBox.Ok)
    msgBox.exec_()

if __name__=="__main__":
    pass



import os
import numpy

from PyQt5.QtWidgets import QMessageBox

from orangewidget import gui, widget
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui, congruence
from orangewidget import gui as orangegui

from comsyl.autocorrelation.CompactAFReader import CompactAFReader

class OWModesFileReader(widget.OWWidget):
    name = "Modes from COMSYL File"
    description = "Utility: COMSYL Modes File Reader"
    icon = "icons/file_reader.png"
    maintainer = "Manuel Sanchez del Rio"
    maintainer_email = "srio(@at@)esrf.eu"
    priority = 60
    category = "Utility"
    keywords = ["COMSYL", "coherent modes"]

    outputs = [{"name":"COMSYL modes",
                "type":CompactAFReader,
                "doc":"COMSYL modes",
                "id":"COMSYL modes"},]

    want_main_area = 0

    filename = Setting("")


    def __init__(self):
        super().__init__()

        self.runaction = widget.OWAction("Read COMSYL File", self)
        self.runaction.triggered.connect(self.read_file)
        self.addAction(self.runaction)

        self.setFixedWidth(590)
        self.setFixedHeight(150)

        left_box_1 = oasysgui.widgetBox(self.controlArea, "COMSYL File Selection", addSpace=True, orientation="vertical",
                                         width=570, height=70)

        figure_box = oasysgui.widgetBox(left_box_1, "", addSpace=True, orientation="horizontal", width=550, height=35)

        self.le_beam_file_name = oasysgui.lineEdit(figure_box, self, "filename", "COMSYL File Name",
                                                    labelWidth=120, valueType=str, orientation="horizontal")
        self.le_beam_file_name.setFixedWidth(330)

        gui.button(figure_box, self, "...", callback=self.selectFile)

        #gui.separator(left_box_1, height=20)

        button = gui.button(self.controlArea, self, "Read COMSYL File", callback=self.read_file)
        button.setFixedHeight(45)

        gui.rubber(self.controlArea)

    def selectFile(self):
        self.le_beam_file_name.setText(oasysgui.selectFileFromDialog(self, self.filename, "Open COMSYL File"))

    def read_file(self):
        self.setStatusMessage("")
        filename = self.le_beam_file_name.text()
        if not os.path.exists(filename):
            raise Exception("File not found %s"%filename)

        try:
            if congruence.checkFileName(filename):

                try:
                    self.af = CompactAFReader.initialize_from_file(filename)
                except:
                    raise FileExistsError("Error loading COMSYL modes from file: %s"%filename)

        except:
            raise Exception("Failed to read file %s"%filename)

        self.send("COMSYL modes", self.af)


if __name__ == "__main__":
    from PyQt5.QtWidgets import QApplication
    import sys

    a = QApplication(sys.argv)
    ow = OWModesFileReader()

    ow.filename = "/users/srio/COMSYLD/comsyl/comsyl/calculations/septest_cm_new_u18_2m_1h_s2.5.npy"

    ow.show()
    a.exec_()



import numpy
from syned.storage_ring.light_source import LightSource
# from syned.storage_ring.electron_beam import ElectronBeam
from syned.storage_ring.empty_light_source import EmptyLightSource
from syned.storage_ring.magnetic_structures.undulator import Undulator

from wofry.beamline.decorators import LightSourceDecorator
# from wofryimpl.propagator.util.undulator_coherent_mode_decomposition_1d import UndulatorCoherentModeDecomposition1D
#
# from wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D

from comsyl.autocorrelation.CompactAFReader import CompactAFReader

class WOLightSourceCOMSYL(LightSource, LightSourceDecorator):
    def __init__(self,
                 name                = "Undefined",
                 filename="",  # comsyl result filename
                 # electron_beam       = None,
                 # magnetic_structure  = None,
                 # undulator_coherent_mode_decomposition_1d = None,
                 # dimension           = 2,
                 mode_index=None, # for wavefront retrieving
                 normalize_with_eigenvalue=1, # for wavefront retrieving (integer)
                 ):

        electron_beam = EmptyLightSource(name=name)
        magnetic_structure = None # Undulator(K_vertical=undulator_coherent_mode_decomposition_1d.K,
                                       # period_length=undulator_coherent_mode_decomposition_1d.undulator_period,
                                       # number_of_periods=undulator_coherent_mode_decomposition_1d.undulator_nperiods)

        LightSource.__init__(self, name=name,
                             electron_beam=electron_beam,
                             magnetic_structure=magnetic_structure)

        self.filename = filename
        self.af_oasys = None
        self._dimension =  2
        self.dimension = 2
        self.mode_index = mode_index
        self.normalize_with_eigenvalue = normalize_with_eigenvalue
        self._set_support_text([
                    # ("name"      ,           "to define ", "" ),
                    ("dimension"      , "dimension ", "" ),
                    ("filename", "filename ", ""),
            ] )

    def set_mode_index(self, mode_index):
        self.mode_index = mode_index

    def get_mode_index(self):
        return self.mode_index

    def get_filename(self):
        return self.filename

    def get_dimension(self):
        return self._dimension

    def load_comsyl_file(self):
        self.af_oasys = CompactAFReader.initialize_from_file(self.filename)

    # from Wofry Decorator
    def get_wavefront(self):
        if self.af_oasys is None: self.load_comsyl_file()
        return self.af_oasys.get_wavefront(mode_index=self.mode_index,
                                           normalize_with_eigenvalue=self.normalize_with_eigenvalue)

    def to_python_code(self, do_plot=True, add_import_section=False):

        txt = ""

        txt += "#"
        txt += "\n# create output_wavefront\n#"
        txt += "\n#"

        if self._dimension == 2:
            txt += "\nfrom comsyl.autocorrelation.CompactAFReader import CompactAFReader"
        else:
            raise Exception("Not implemented")

        txt += '\nfilename = "%s"' % self.filename
        txt += "\naf_oasys = CompactAFReader.initialize_from_file(filename)"
        txt += "\n\nmode_index = %d"  % self.mode_index
        txt += "\noutput_wavefront = af_oasys.get_wavefront(mode_index,normalize_with_eigenvalue=%d)" % self.normalize_with_eigenvalue

        return txt


if __name__ == "__main__":
    filename = "/users/srio/Oasys/id18_ebs_u18_2500mm_s3.0.npz"
    pp = WOLightSourceCOMSYL(name="tmp", filename=filename, mode_index=3)

    print(">>>>> Dimension: ", pp.get_dimension())
    print(">>>>> filename: ", pp.get_filename())
    print(">>>>> code: \n", pp.to_python_code())

    wf = pp.get_wavefront()
    from srxraylib.plot.gol import plot_image
    plot_image(wf.get_intensity(), wf.get_coordinate_x(), wf.get_coordinate_y())



from oasys.widgets import widget

from orangewidget import gui
from orangewidget.settings import Setting

from PyQt5.QtWidgets import QApplication
from PyQt5.QtCore import QRect

from oasys.widgets.gui import ConfirmDialog

class OWComsylWidget(widget.OWWidget):

    want_main_area=1

    is_automatic_run = Setting(True)

    error_id = 0
    warning_id = 0
    info_id = 0

    MAX_WIDTH = 1320
    MAX_HEIGHT = 700

    CONTROL_AREA_WIDTH = 405
    TABS_AREA_HEIGHT = 560

    comsyl_live_propagation_mode = "Unknown"

    def __init__(self, show_general_option_box=True, show_automatic_box=True):
        super().__init__()

        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width()*0.05),
                               round(geom.height()*0.05),
                               round(min(geom.width()*0.98, self.MAX_WIDTH)),
                               round(min(geom.height()*0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())

        self.controlArea.setFixedWidth(self.CONTROL_AREA_WIDTH)

        self.general_options_box = gui.widgetBox(self.controlArea, "General Options", addSpace=True, orientation="horizontal")
        self.general_options_box.setVisible(show_general_option_box)

        if show_automatic_box :
            gui.checkBox(self.general_options_box, self, 'is_automatic_run', 'Automatic Execution')

    def callResetSettings(self):
        if ConfirmDialog.confirmed(parent=self, message="Confirm Reset of the Fields?"):
            try:
                self.resetSettings()
            except:
                pass



#
# TODO:
#      - Add progress bar


from PyQt5.QtWidgets import QApplication
from PyQt5.QtCore import QRect

from PyQt5 import  QtGui, QtWidgets

from silx.gui.plot import PlotWindow, Plot2D
from silx.gui.plot.StackView import StackViewMainWindow

import numpy

from orangewidget import widget as orange_widget

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import widget
from oasys.widgets import congruence
from oasys.widgets import gui as oasysgui

from comsyl.autocorrelation.CompactAFReader import CompactAFReader


# from oasys.util.oasys_util import TriggerIn, TriggerOut, EmittingStream

class OWModesLoader(widget.OWWidget):
    name = "Modes loader"
    id = "orangecontrib.comsyl.widgets.applications.AFViewer"
    description = ""
    icon = "icons/loader.png"
    author = ""
    maintainer_email = "srio@esrf.fr"
    priority = 40
    category = ""
    keywords = ["COMSYL", "coherent modes"]

    outputs = [
                {"name": "COMSYL modes",
                "type": CompactAFReader,
                "doc": "COMSYL modes",
                "id": "COMSYL modes"},
                ]


    IMAGE_WIDTH = 760
    IMAGE_HEIGHT = 545
    MAX_WIDTH = 1320
    MAX_HEIGHT = 700
    CONTROL_AREA_WIDTH = 405

    beam_file_name = Setting("/users/srio/COMSYLD/comsyl/comsyl/calculations/septest_cm_new_u18_2m_1h_s2.5.h5")

    TYPE_PRESENTATION = Setting(0) # 0=intensity, 1=real, 2=phase

    INDIVIDUAL_MODES = Setting(False)

    MODE_INDEX = Setting(0)

    def __init__(self):

        super().__init__()

        self._input_available = False
        self.af = None


        self.runaction = orange_widget.OWAction("Load COMSYL files", self)
        self.runaction.triggered.connect(self.read_file)
        self.addAction(self.runaction)


        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width()*0.05),
                               round(geom.height()*0.05),
                               round(min(geom.width()*0.98, self.MAX_WIDTH)),
                               round(min(geom.height()*0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())


        self.controlArea.setFixedWidth(self.CONTROL_AREA_WIDTH)

        self.build_left_panel()

        self.process_showers()

        gui.rubber(self.controlArea)

        self.main_tabs = gui.tabWidget(self.mainArea)
        plot_tab = gui.createTabPage(self.main_tabs, "Results")
        info_tab = gui.createTabPage(self.main_tabs, "Info")

        self.tab = []
        self.tabs = gui.tabWidget(plot_tab)
        self.info = gui.tabWidget(info_tab)
        self.tab_titles = []
        self.initialize_tabs()

        # info tab
        self.comsyl_output = QtWidgets.QTextEdit()
        self.comsyl_output.setReadOnly(True)

        out_box = gui.widgetBox(self.info, "COMSYL file info", addSpace=True, orientation="horizontal")
        out_box.layout().addWidget(self.comsyl_output)

        self.comsyl_output.setFixedHeight(self.IMAGE_HEIGHT)
        self.comsyl_output.setFixedWidth(self.IMAGE_WIDTH)



    def initialize_tabs(self):

        size = len(self.tab)
        indexes = range(0, size)

        for index in indexes:
            self.tabs.removeTab(size-1-index)

        self.tab = []
        self.plot_canvas = []

        for index in range(0, len(self.tab_titles)):
            self.tab.append(gui.createTabPage(self.tabs, self.tab_titles[index]))
            self.plot_canvas.append(None)

        for tab in self.tab:
            tab.setFixedHeight(self.IMAGE_HEIGHT)
            tab.setFixedWidth(self.IMAGE_WIDTH)

    def write_std_out(self, text):
        cursor = self.comsyl_output.textCursor()
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertText(text)
        self.comsyl_output.setTextCursor(cursor)
        self.comsyl_output.ensureCursorVisible()

    def build_left_panel(self):

        left_box_1 = oasysgui.widgetBox(self.controlArea, "Files Selection", addSpace=True, orientation="vertical")

        figure_box = oasysgui.widgetBox(left_box_1, "", addSpace=True, orientation="horizontal") #width=550, height=50)
        self.le_beam_file_name = oasysgui.lineEdit(figure_box, self, "beam_file_name", "COMSYL File:",
                                                    labelWidth=90, valueType=str, orientation="horizontal")
        gui.button(figure_box, self, "...", callback=self.selectFile)
        gui.separator(left_box_1)

        button = gui.button(self.controlArea, self, "Read COMSYL File", callback=self.read_file)
        button.setFixedHeight(45)



        left_box_2 = oasysgui.widgetBox(self.controlArea, "Display mode", addSpace=True, orientation="vertical")

        gui.comboBox(left_box_2, self, "TYPE_PRESENTATION",
                    label="Display magnitude ", addSpace=False,
                    items=['intensity','modulus','real part','imaginary part','angle [rad]'],
                    valueType=int, orientation="horizontal", callback=self.do_plot)
        gui.separator(left_box_1, height=20)

        gui.comboBox(left_box_2, self, "INDIVIDUAL_MODES",
                    label="Access individual modes ", addSpace=False,
                    items=['No [Fast]','Yes [Slow, memory hungry]',],
                    valueType=int, orientation="horizontal", callback=self.do_plot)
        gui.separator(left_box_1, height=20)



        mode_index_box = oasysgui.widgetBox(left_box_2, "", addSpace=True, orientation="horizontal", ) #width=550, height=50)
        oasysgui.lineEdit(mode_index_box, self, "MODE_INDEX",
                    label="Plot mode ", addSpace=False,
                    valueType=int, orientation="horizontal", labelWidth=150,
                    callback=self.do_plot)
        gui.button(mode_index_box, self, "+1", callback=self.increase_mode_index)

        button = gui.button(self.controlArea, self, "PLOT COMSYL data", callback=self.do_plot)
        button.setFixedHeight(45)


    def increase_mode_index(self):
        if self.MODE_INDEX+1 >= self.af.number_of_modes():
            raise Exception("Mode index %d not available"%(self.MODE_INDEX+1))
        self.MODE_INDEX += 1
        self.do_plot()


    def set_selected_file(self,filename):
        self.le_beam_file_name.setText(filename)

    def selectFile(self):
        filename = oasysgui.selectFileFromDialog(self,
                previous_file_path=self.beam_file_name, message="Open COMSYL File [*.npy or *.npz or *.h5]",
                start_directory=".", file_extension_filter="*.*")

        self.le_beam_file_name.setText(filename)

    def read_file(self):
        self.setStatusMessage("")
        filename = self.le_beam_file_name.text()
        try:
            if congruence.checkFileName(filename):

                # # just in case old file is open
                # try:
                #     self.af.close_h5_file()
                # except:
                #     pass

                try:
                    self.af = CompactAFReader.initialize_from_file(filename)
                    self._input_available = True
                    self.write_std_out(self.af.info(list_modes=False))
                    self.main_tabs.setCurrentIndex(1)
                    self.initialize_tabs()

                    self.send("COMSYL modes", self.af)
                except:
                    raise FileExistsError("Error loading COMSYL modes from file: %s"%filename)

        except:
            raise Exception("Failed to read file %s"%filename)

    def _square_modulus(self,array1):
        return (numpy.absolute(array1))**2

    def plot_data2D(self, data2D, dataX, dataY, plot_canvas_index, title="", xtitle="", ytitle=""):

        xmin = numpy.min(dataX)
        xmax = numpy.max(dataX)
        ymin = numpy.min(dataY)
        ymax = numpy.max(dataY)

        origin = (xmin, ymin)
        scale = (abs((xmax-xmin)/len(dataX)), abs((ymax-ymin)/len(dataY)))

        data_to_plot = data2D.T

        colormap = {"name":"temperature", "normalization":"linear", "autoscale":True, "vmin":0, "vmax":0, "colors":256}

        self.plot_canvas[plot_canvas_index] = Plot2D()

        self.plot_canvas[plot_canvas_index].resetZoom()
        self.plot_canvas[plot_canvas_index].setXAxisAutoScale(True)
        self.plot_canvas[plot_canvas_index].setYAxisAutoScale(True)
        self.plot_canvas[plot_canvas_index].setGraphGrid(False)
        self.plot_canvas[plot_canvas_index].setKeepDataAspectRatio(True)
        self.plot_canvas[plot_canvas_index].yAxisInvertedAction.setVisible(False)

        self.plot_canvas[plot_canvas_index].setXAxisLogarithmic(False)
        self.plot_canvas[plot_canvas_index].setYAxisLogarithmic(False)

        self.plot_canvas[plot_canvas_index].getMaskAction().setVisible(False)
        self.plot_canvas[plot_canvas_index].getRoiAction().setVisible(False)
        self.plot_canvas[plot_canvas_index].getColormapAction().setVisible(True)
        self.plot_canvas[plot_canvas_index].setKeepDataAspectRatio(False)

        self.plot_canvas[plot_canvas_index].addImage(numpy.array(data_to_plot),
                                                     legend="",
                                                     scale=scale,
                                                     origin=origin,
                                                     colormap=colormap,
                                                     replace=True)


        self.plot_canvas[plot_canvas_index].setGraphXLabel(xtitle)
        self.plot_canvas[plot_canvas_index].setGraphYLabel(ytitle)
        self.plot_canvas[plot_canvas_index].setGraphTitle(title)


        self.tab[plot_canvas_index].layout().addWidget(self.plot_canvas[plot_canvas_index])

    def do_plot(self):

        old_tab_index = self.tabs.currentIndex()

        try:
            for i in range(len(self.tab_titles)):
                self.tab[i].layout().removeItem(self.tab[i].layout().itemAt(0))
        except:
            pass

        if self.INDIVIDUAL_MODES:
            self.tab_titles = ["SPECTRUM",
                               "CUMULATED SPECTRUM",
                               "SPECTRAL DENSITY (INTENSITY)",
                               "SPECTRAL INTENSITY FROM MODES",
                               "REFERENCE ELECRON DENSITY",
                               "REFERENCE UNDULATOR WAVEFRONT",
                               "INDIVIDUAL MODES",
                               ]
        else:
            self.tab_titles = ["SPECTRUM",
                               "CUMULATED SPECTRUM",
                               "SPECTRAL DENSITY (INTENSITY)",
                               "REFERENCE ELECRON DENSITY",
                               "REFERENCE UNDULATOR WAVEFRONT",
                               "MODE INDEX: %d" % self.MODE_INDEX,
                               ]

        self.initialize_tabs()

        if self.TYPE_PRESENTATION == 0:
            myprocess = self._square_modulus
            title0 = "Intensity of eigenvalues"
            title1 = "Intensity of eigenvector"
        if self.TYPE_PRESENTATION == 1:
            myprocess = numpy.absolute
            title0 = "Modulus of eigenvalues"
            title1 = "Modulus of eigenvector"
        elif self.TYPE_PRESENTATION == 2:
            myprocess = numpy.real
            title0 = "Real part of eigenvalues"
            title1 = "Real part of eigenvector"
        elif self.TYPE_PRESENTATION == 3:
            myprocess = numpy.imag
            title0 = "Imaginary part of eigenvalues"
            title1 = "Imaginary part of eigenvectos"
        elif self.TYPE_PRESENTATION == 4:
            myprocess = numpy.angle
            title0 = "Angle of eigenvalues [rad]"
            title1 = "Angle of eigenvector [rad]"

        if self._input_available:
            x_values = numpy.arange(self.af.number_modes())
            x_label = "Mode index"
            y_label =  "Occupation"


            xx = self.af.x_coordinates()
            yy = self.af.y_coordinates()

            xmin = numpy.min(xx)
            xmax = numpy.max(xx)
            ymin = numpy.min(yy)
            ymax = numpy.max(yy)

        else:
            raise Exception("Nothing to plot")

        #
        # plot spectrum
        #
        tab_index = 0
        self.plot_canvas[tab_index] = PlotWindow(parent=None,
                                                         backend=None,
                                                         resetzoom=True,
                                                         autoScale=False,
                                                         logScale=True,
                                                         grid=True,
                                                         curveStyle=True,
                                                         colormap=False,
                                                         aspectRatio=False,
                                                         yInverted=False,
                                                         copy=True,
                                                         save=True,
                                                         print_=True,
                                                         control=False,
                                                         position=True,
                                                         roi=False,
                                                         mask=False,
                                                         fit=False)


        self.tab[tab_index].layout().addWidget(self.plot_canvas[tab_index])

        self.plot_canvas[tab_index].setDefaultPlotLines(True)
        self.plot_canvas[tab_index].setXAxisLogarithmic(False)
        self.plot_canvas[tab_index].setYAxisLogarithmic(False)
        self.plot_canvas[tab_index].setGraphXLabel(x_label)
        self.plot_canvas[tab_index].setGraphYLabel(y_label)
        self.plot_canvas[tab_index].addCurve(x_values, numpy.abs(self.af.occupation_array()), title0, symbol='', xlabel="X", ylabel="Y", replace=False) #'+', '^', ','

        self.tab[tab_index].layout().addWidget(self.plot_canvas[tab_index])
        #
        # plot cumulated spectrum
        #
        tab_index += 1
        self.plot_canvas[tab_index] = PlotWindow(parent=None,
                                                         backend=None,
                                                         resetzoom=True,
                                                         autoScale=False,
                                                         logScale=True,
                                                         grid=True,
                                                         curveStyle=True,
                                                         colormap=False,
                                                         aspectRatio=False,
                                                         yInverted=False,
                                                         copy=True,
                                                         save=True,
                                                         print_=True,
                                                         control=False,
                                                         position=True,
                                                         roi=False,
                                                         mask=False,
                                                         fit=False)


        self.tab[tab_index].layout().addWidget(self.plot_canvas[tab_index])

        self.plot_canvas[tab_index].setDefaultPlotLines(True)
        self.plot_canvas[tab_index].setXAxisLogarithmic(False)
        self.plot_canvas[tab_index].setYAxisLogarithmic(False)
        self.plot_canvas[tab_index].setGraphXLabel(x_label)
        self.plot_canvas[tab_index].setGraphYLabel("Cumulated occupation")
        self.plot_canvas[tab_index].addCurve(x_values, self.af.cumulated_occupation_array(), "Cumulated occupation", symbol='', xlabel="X", ylabel="Y", replace=False) #'+', '^', ','

        self.plot_canvas[tab_index].setGraphYLimits(0.0,1.0)

        self.tab[tab_index].layout().addWidget(self.plot_canvas[tab_index])

        #
        # plot spectral density
        #
        tab_index += 1
        image = myprocess( (self.af.spectral_density()))
        self.plot_data2D( image,
                1e6*self.af.x_coordinates(),
                1e6*self.af.y_coordinates(),
                tab_index,
                title="Spectral Density (Intensity)",
                xtitle="X [um] (%d pixels)"%(image.shape[0]),
                ytitle="Y [um] (%d pixels)"%(image.shape[1]))


        #
        # plot spectral density from modes
        #
        if self.INDIVIDUAL_MODES:
            tab_index += 1
            image = myprocess( (self.af.intensity_from_modes()))
            self.plot_data2D( image,
                    1e6*self.af.x_coordinates(),
                    1e6*self.af.y_coordinates(),
                    tab_index,
                    title="Spectral Density (Intensity)",
                    xtitle="X [um] (%d pixels)"%(image.shape[0]),
                    ytitle="Y [um] (%d pixels)"%(image.shape[1]))


        #
        # plot reference electron density
        #
        tab_index += 1
        image = numpy.abs( self.af.reference_electron_density() )**2  #TODO: Correct? it is complex...
        self.plot_data2D( image,
                1e6*self.af.x_coordinates(),
                1e6*self.af.y_coordinates(),
                tab_index,
                title="Reference electron density",
                xtitle="X [um] (%d pixels)"%(image.shape[0]),
                ytitle="Y [um] (%d pixels)"%(image.shape[1]))

        #
        # plot reference undulator radiation
        #
        tab_index += 1
        image = self.af.reference_undulator_radiation()[0,:,:,0]   #TODO: Correct? is polarized?
        self.plot_data2D( image,
                1e6*self.af.x_coordinates(),
                1e6*self.af.y_coordinates(),
                tab_index,
                title="Reference undulator radiation",
                xtitle="X [um] (%d pixels)"%(image.shape[0]),
                ytitle="Y [um] (%d pixels)"%(image.shape[1]))

        #
        # plot all modes
        #

        if self.INDIVIDUAL_MODES:
            tab_index += 1
            dim0_calib = (0, 1)
            dim1_calib = (1e6*yy[0], 1e6*(yy[1]-yy[0]))
            dim2_calib = (1e6*xx[0], 1e6*(xx[1]-xx[0]))


            colormap = {"name":"temperature", "normalization":"linear", "autoscale":True, "vmin":0, "vmax":0, "colors":256}

            self.plot_canvas[tab_index] = StackViewMainWindow()
            self.plot_canvas[tab_index].setGraphTitle(title1)
            self.plot_canvas[tab_index].setLabels(["Mode number",
                                           "Y index from %4.2f to %4.2f um"%(1e6*ymin,1e6*ymax),
                                           "X index from %4.2f to %4.2f um"%(1e6*xmin,1e6*xmax),
                                           ])
            self.plot_canvas[tab_index].setColormap(colormap=colormap)

            self.plot_canvas[tab_index].setStack( myprocess(numpy.swapaxes(self.af.modes(),2,1)),
                                          calibrations=[dim0_calib, dim1_calib, dim2_calib] )

            self.tab[tab_index].layout().addWidget(self.plot_canvas[tab_index])
        else:
            tab_index += 1
            image = myprocess( (self.af.mode(self.MODE_INDEX)))
            self.plot_data2D( image,
                    1e6*self.af.x_coordinates(),
                    1e6*self.af.y_coordinates(),
                    tab_index,
                    title="Mode %d"%self.MODE_INDEX,
                    xtitle="X [um] (%d pixels)"%(image.shape[0]),
                    ytitle="Y [um] (%d pixels)"%(image.shape[1]))

        #
        try:
            self.tabs.setCurrentIndex(old_tab_index)
        except:
            pass


    def get_doc(self):
        pass

if __name__ == '__main__':

    app = QApplication([])
    ow = OWModesLoader()

    filename = "/scisoft/users/srio/COMSYL-SLURM/comsyl/comsyl-submit/calculations/id18_ebs_u18_2500mm_s6.0.npz"
    ow.set_selected_file(filename)
    ow.read_file()
    ow.do_plot()

    ow.show()
    app.exec_()
    ow.saveSettings()



class ComsylPreprocessorData(object):
    def __init__(self,beamline_pickle_file=""):
        self._beamline_pickle_file=beamline_pickle_file

    def get_beamline_pickle_file(self,filename):
        self._beamline_pickle_file = filename

    def get_beamline_pickle_file(self):
        return self._beamline_pickle_file


if __name__=="__main__":
    pass


import os, sys

from PyQt5 import QtGui, QtWidgets
from PyQt5.QtGui import QPalette, QColor, QFont
from PyQt5.QtWidgets import QApplication, QFileDialog

from PyQt5.QtGui import QIntValidator, QDoubleValidator

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence

from comsyl.autocorrelation.CompactAFReader import CompactAFReader

from orangecontrib.comsyl.widgets.gui.ow_comsyl_widget import OWComsylWidget
from orangecontrib.comsyl.util.preprocessor import ComsylPreprocessorData
from orangecontrib.comsyl.util.python_script import PythonConsole
from orangecontrib.comsyl.util.messages import showConfirmMessage


class OWComsylPropagateBeamline(OWComsylWidget):

    name = "Propagate Beamline Script"
    description = "COMSYL Propagate Beamline"
    icon = "icons/propagator.png"
    maintainer = "Manuel Sanchez del Rio"
    maintainer_email = "srio(@at@)esrf.eu"
    priority = 46
    category = ""
    keywords = ["COMSYL", "coherent modes"]

    inputs = [("COMSYL modes" , CompactAFReader, "setCompactAFReader" ),
              ("COMSYL preprocessor beamline" , ComsylPreprocessorData, "setPreprocessor" ),]

    outputs = [{"name":"COMSYL modes",
                "type":CompactAFReader,
                "doc":"COMSYL modes",
                "id":"COMSYL modes"} ]

    COMSYL_AF_FILE = ""
    BL_PICKLE_FILE = ""
    MODE_INDEX = Setting(2) # maxumim mode index
    REFERENCE_SOURCE = Setting(0)
    DIRECTORY_NAME = "tmp_comsyl_propagation"
    PYTHON_INTERPRETER = sys.executable


    IMAGE_WIDTH = 890
    IMAGE_HEIGHT = 680

    def __init__(self, show_automatic_box=True):
        super().__init__(show_automatic_box=show_automatic_box)

        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Refresh Script", callback=self.refresh_script)
        font = QFont(button.font())
        font.setBold(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Blue'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)

        button = gui.button(button_box, self, "Reset Fields", callback=self.callResetSettings)
        font = QFont(button.font())
        font.setItalic(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Red'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)
        button.setFixedWidth(150)

        gui.separator(self.controlArea)

        gen_box = oasysgui.widgetBox(self.controlArea, "COMSYL Beamline Propagation", addSpace=False, orientation="vertical", height=530, width=self.CONTROL_AREA_WIDTH-5)


        figure_box0 = oasysgui.widgetBox(gen_box, "", addSpace=True, orientation="horizontal")
        self.id_comsyl_af_file = oasysgui.lineEdit(figure_box0, self, "COMSYL_AF_FILE", "Comsyl File with Modes:",
                                                    labelWidth=90, valueType=str, orientation="horizontal")
        gui.button(figure_box0, self, "...", callback=self.select_comsyl_af_file)


        figure_box = oasysgui.widgetBox(gen_box, "", addSpace=True, orientation="horizontal")
        self.id_bl_pickle_file = oasysgui.lineEdit(figure_box, self, "BL_PICKLE_FILE", "BL Pickle File:",
                                                    labelWidth=90, valueType=str, orientation="horizontal")
        gui.button(figure_box, self, "...", callback=self.select_bl_pickle_file)

        oasysgui.lineEdit(gen_box, self, "MODE_INDEX",
                    label="Maximum Mode index", addSpace=False,
                    valueType=int, validator=QIntValidator(), orientation="horizontal", labelWidth=150)

        oasysgui.lineEdit(gen_box, self, "DIRECTORY_NAME", "Temporal Directory", labelWidth=160, valueType=str, orientation="horizontal")
        oasysgui.lineEdit(gen_box, self, "PYTHON_INTERPRETER", "Python interpreter", labelWidth=160, valueType=str, orientation="horizontal")


        tabs_setting = oasysgui.tabWidget(self.mainArea)
        tabs_setting.setFixedHeight(self.IMAGE_HEIGHT)
        tabs_setting.setFixedWidth(self.IMAGE_WIDTH)

        tab_scr = oasysgui.createTabPage(tabs_setting, "Python Script")
        tab_out = oasysgui.createTabPage(tabs_setting, "System Output")

        self.pythonScript = oasysgui.textArea(readOnly=False)
        self.pythonScript.setStyleSheet("background-color: white; font-family: Courier, monospace;")
        self.pythonScript.setMaximumHeight(self.IMAGE_HEIGHT - 250)

        script_box = oasysgui.widgetBox(tab_scr, "", addSpace=False, orientation="vertical", height=self.IMAGE_HEIGHT - 10, width=self.IMAGE_WIDTH - 10)
        script_box.layout().addWidget(self.pythonScript)

        console_box = oasysgui.widgetBox(script_box, "", addSpace=True, orientation="vertical",
                                          height=150, width=self.IMAGE_WIDTH - 10)

        self.console = PythonConsole(self.__dict__, self)
        console_box.layout().addWidget(self.console)

        self.shadow_output = oasysgui.textArea()

        out_box = oasysgui.widgetBox(tab_out, "System Output", addSpace=True, orientation="horizontal", height=self.IMAGE_WIDTH - 45)
        out_box.layout().addWidget(self.shadow_output)


        button_box = oasysgui.widgetBox(tab_scr, "", addSpace=True, orientation="horizontal")

        gui.button(button_box, self, "Run Script", callback=self.execute_script, height=40)
        gui.button(button_box, self, "Save Script to File", callback=self.save_script, height=40)

        #############################

        # self.refresh_script()
    #

    def select_comsyl_af_file(self):
        self.id_comsyl_af_file.setText(oasysgui.selectFileFromDialog(self,
                        self.COMSYL_AF_FILE, "Select Input File",
                        file_extension_filter="COMSYL Files (*.npz)"))


    def select_bl_pickle_file(self):
        self.id_bl_pickle_file.setText(oasysgui.selectFileFromDialog(self,
                        self.BL_PICKLE_FILE, "Select Input File",
                        file_extension_filter="COMSYL Beamline Pickle Files (*.p)"))


    def setCompactAFReader(self, af):
        if not af is None:
            self.COMSYL_AF_FILE = af._af._io.fromFile()
            self.refresh_script()

    def setPreprocessor(self, data):
        try:
            self.BL_PICKLE_FILE = data.get_beamline_pickle_file()
            self.refresh_script()
        except:
            pass

    def execute_script(self):
        if showConfirmMessage(message = "Do you confirm launching a COMSYL propagation?",
                              informative_text="This is a long and resource-consuming process: launching it within the OASYS environment is highly discouraged." + \
                                               "The suggested solution is to save the script into a file and to launch it in a different environment."):
            self._script = str(self.pythonScript.toPlainText())
            self.console.write("\nRunning script:\n")
            self.console.push("exec(_script)")
            self.console.new_prompt(sys.ps1)

    def save_script(self):
        file_name = QFileDialog.getSaveFileName(self, "Save File to Disk", os.getcwd(), filter='*.py')[0]

        if not file_name is None:
            if not file_name.strip() == "":
                file = open(file_name, "w")
                file.write(str(self.pythonScript.toPlainText()))
                file.close()

                QtWidgets.QMessageBox.information(self, "QMessageBox.information()",
                                              "File " + file_name + " written to disk",
                                              QtWidgets.QMessageBox.Ok)

    def refresh_script(self):
        dd = {"COMSYL_AF_FILE": self.COMSYL_AF_FILE,
              "BL_PICKLE_FILE": self.BL_PICKLE_FILE,
              "DIRECTORY_NAME": self.DIRECTORY_NAME,
              "PYTHON_INTERPRETER": self.PYTHON_INTERPRETER,
              "MODE_INDEX": self.MODE_INDEX,
              }

        self.pythonScript.setText(self.script_template().format_map(dd))

    def script_template(self):
        return """import pickle
from comsyl.waveoptics.ComsylWofryBeamline import ComsylWofryBeamline
from comsyl.waveoptics.SRWAdapter import ComsylSRWBeamline
from comsyl.autocorrelation.CompactAFReader import CompactAFReader

comsyl_beamline  = pickle.load(open("{BL_PICKLE_FILE}","rb"))

filename = "{COMSYL_AF_FILE}"
af_oasys = CompactAFReader.initialize_from_file(filename)
af_comsyl = af_oasys.get_af()

# **source position correction**
source_position=af_comsyl.info().sourcePosition()
if source_position == "entrance":
    source_offset = af_comsyl._undulator.length() * 0.5
elif source_position == "center":
    source_offset = 0.0
else:
    raise Exception("Unhandled source position")
print("Using source position entrance z=%f" % source_offset)
comsyl_beamline.add_undulator_offset(source_offset)


af_propagated = comsyl_beamline.propagate_af(af_comsyl,
             directory_name="{DIRECTORY_NAME}",
             af_output_file_root="{DIRECTORY_NAME}/propagated_beamline",
             maximum_mode={MODE_INDEX},
             python_to_be_used="{PYTHON_INTERPRETER}")

#rediagonalization   **uncomment to proceed**
#af_propagated.diagonalizeModes({MODE_INDEX})
#af_propagated.save("{DIRECTORY_NAME}/rediagonalized")

"""


if __name__ == '__main__':

    from PyQt5.QtWidgets import QApplication

    app = QApplication([])
    ow = OWComsylPropagateBeamline()

    ow.COMSYL_AF_FILE = "/scisoft/users/glass/Documents/sources/Orange-SRW/comsyl/calculations/cs_new_u18_2m_1h_s2.5.npz"
    ow.BL_PICKLE_FILE = "/scisoft/xop2.4/extensions/shadowvui/shadow3-scripts/HIGHLIGHTS/bl.p"
    ow.refresh_script()

    ow.show()
    app.exec_()
    ow.saveSettings()



import h5py
import numpy

try:
    from comsyl.scripts.CompactAFReader import CompactAFReader
except:
    try:
        from CompactAFReader import CompactAFReader
    except:
        raise ImportError("Cannt find CompactAFReader")


def AF_to_h5(filename_without_extension,filename_out='comsyl_test.h5',max_number_of_modes=None,write_intensity=True):


    reader = CompactAFReader(filename_without_extension)

    print("File %s:" % filename_in)
    print("contains")
    print("%i modes" % reader.number_modes())
    print("on the grid")
    print("x: from %e to %e" % (reader.x_coordinates().min(), reader.x_coordinates().max()))
    print("y: from %e to %e" % (reader.y_coordinates().min(), reader.y_coordinates().max()))
    print("calculated at %f eV" % reader.photon_energy())
    print("with total intensity in (maybe improper) normalization: %e" % reader.total_intensity().real.sum())



    print("Occupation and max abs value of the mode")

    x = reader.x_coordinates()
    y = reader.y_coordinates()


    if max_number_of_modes == None:
        max_number_of_modes = reader.number_modes()

    eigenvalues = numpy.zeros(max_number_of_modes,dtype=complex)

    # TODO change the order - check with the viewer
    mystack = numpy.zeros((max_number_of_modes,y.size,x.size),dtype=complex)

    for i_mode in range(max_number_of_modes):
        eigenvalues[i_mode] = reader.occupation(i_mode)
        mode = reader.mode(i_mode)
        mystack[i_mode,:,:] = mode.T



    f = h5py.File(filename_out, 'w')

    f['converted_from_filename'] = filename_in
    f['number_of_modes'] = max_number_of_modes
    f['eigenvalues'] = eigenvalues
    f['x'] = x
    f['y'] = y
    f['modes'] = mystack

    if write_intensity:
        mystack_intensity = numpy.absolute(mystack)**2
        f['modes_intensity'] = mystack_intensity
        # delta = (x[1] - x[0]) *(y[1] - y[0])
        # for i in range(reader.number_modes()):
        #     print("mode: %d integral: %f"%(i,mystack_intensity[i,:,:].sum()*delta))

    f.close()
    print("File written to disk",filename_out)



if __name__ == "__main__":

    path_in = "/scisoft/users/srio/comsyl_srio/calculations/"
    path_out = path_in

    files = ['new_u18_2m_1h_s2.5.npy'] # ,'ph3_u18_3_17keV_s1.3.npy']

    for filename_in in files:
        print("Converting file: ",path_in+filename_in)
        filename_without_extension = ('.').join(filename_in.split('.')[:-1])
        AF_to_h5(path_in+filename_without_extension,
                 path_out+filename_without_extension+'_100modes.h5',
                 max_number_of_modes=100,write_intensity=True)




import sys
import os


def read_json(json_name):
    json_text = open(json_name).read()
    json_dict = eval(json_text)
    json = sorted(json_dict.items(),
                  key=lambda x: json_text.index('"{}"'.format(x[0])))
    json_lowercase = json
    return json_lowercase


def create_settings(json):
    settings = ""
    for name, value in json:
        if isinstance(value, str):
            settings += '    {} = Setting("{}")\n'.format(name, value)
        elif isinstance(value, list):
            settings += '    {} = Setting({})\n'.format(name, value[0])
        else:
            settings += '    {} = Setting({})\n'.format(name, value)
    return settings

def create_calc_args_default(json):
    settings = ""
    i = -1
    for name, value in json:
        i += 1
        if isinstance(value, str):
            settings += '{}="{}"'.format(name, value).rstrip('\n')
        elif isinstance(value, list):
            settings += '{}={}'.format(name, value[0]).rstrip('\n')
        else:
            settings += '{}={}\n'.format(name, value).rstrip('\n')
        if i < (len(json)-1):
           settings += ','
    return settings

def create_calc_args(json):
    calc_args = ""
    i = -1
    for name, value in json:
        i += 1
        calc_args += '{}=self.{}'.format(name, name)
        if i < (len(json)-1):
           calc_args += ','
    return calc_args


def create_controls(json):
    controls = ""
    controls += '        box0 = gui.widgetBox(self.controlArea, " ",orientation="horizontal") \n'
    controls += '        #widget buttons: compute, set defaults, help\n'
    controls += '        gui.button(box0, self, "Compute", callback=self.compute)\n'
    controls += '        gui.button(box0, self, "Defaults", callback=self.defaults)\n'
    controls += '        gui.button(box0, self, "Help", callback=self.get_doc)\n'
    controls += '        self.process_showers()\n'


    controls += '        box = gui.widgetBox(self.controlArea, " ",orientation="vertical") \n'
    idx = -1
    controls += '        \n'
    controls += '        \n'
    controls += '        idx = -1 \n'


    for name, value in json:
        idx += 1
        controls += '        \n'
        controls += '        #widget index '+str(idx)+' \n'
        controls += '        idx += 1 \n'
        controls += '        box1 = gui.widgetBox(box) \n'
        if isinstance(value, list):
            controls += list_template.format(name=name,values=str(value[1:]))
        else:
            controls += line_edit_templates[type(value)].format(name=name)

        controls += '        self.show_at(self.unitFlags()[idx], box1) \n'

    return controls


def main():
    json_name = sys.argv[1]
    base = os.path.splitext(json_name)[0]
    py_name =  base + ".py"

    if os.path.exists(py_name):
        print("file overwritten: "+py_name+"\n")
    else:
        print("file written: "+py_name+"\n")

    json = read_json(json_name)
    widget_name = base
    widget_class_name = widget_id_name = base.replace(" ", "")
    settings = create_settings(json)
    controls = create_controls(json)
    calc_args = create_calc_args(json)
    calc_args_default = create_calc_args_default(json)

    f = open(json_name+'.ext')
    lines = f.readlines()
    f.close()

    labels = lines[0]
    flags = lines[1]
    open(py_name, "wt").write(widget_template.format_map(vars()))



control_template = """        gui.{}(box1, self, "{{name}}",
                     label=self.unitLabels()[idx], addSpace=True"""


str_template = control_template.format("lineEdit") + ")\n"

int_template = control_template.format("lineEdit") + """,
                    valueType=int, validator=QIntValidator())
"""

float_template = control_template.format("lineEdit") + """,
                    valueType=float, validator=QDoubleValidator())
"""

line_edit_templates = {str: str_template, int: int_template,
                       float: float_template}

list_template = control_template.format("comboBox") + """,
                    items={values},
                    valueType=int, orientation="horizontal")
"""



widget_template = """import sys
import numpy as np
from PyQt4.QtGui import QIntValidator, QDoubleValidator, QApplication, QSizePolicy
# from PyMca5.PyMcaIO import specfilewrapper as specfile
from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import widget
import orangecanvas.resources as resources
import sys,os

class OW{widget_class_name}(widget.OWWidget):
    name = "{widget_name}"
    id = "orange.widgets.data{widget_id_name}"
    description = "Application to compute..."
    icon = "icons/{widget_class_name}.png"
    author = "create_widget.py"
    maintainer_email = "srio@esrf.eu"
    priority = 10
    category = ""
    keywords = ["oasysaddontemplate", "{widget_class_name}"]
    outputs = [{{"name": "oasysaddontemplate-data",
                "type": np.ndarray,
                "doc": "transfer numpy arrays"}},
               # another possible output
               # {{"name": "oasysaddontemplate-file",
               #  "type": str,
               #  "doc": "transfer a file"}},
                ]

    # widget input (if needed)
    #inputs = [{{"name": "Name",
    #           "type": type,
    #           "handler": None,
    #           "doc": ""}}]

    want_main_area = False

{settings}

    def __init__(self):
        super().__init__()

{controls}
        gui.rubber(self.controlArea)

    def unitLabels(self):
         return {labels}

    def unitFlags(self):
         return {flags}

    def compute(self):
        dataArray = OW{widget_class_name}.calculate_external_{widget_class_name}({calc_args})

        # if fileName == None:
        #     print("No file to send")
        # else:
        #     self.send("oasysaddontemplate-file",fileName)

        self.send("oasysaddontemplate-data",dataArray)


    def defaults(self):
         self.resetSettings()
         self.compute()
         return

    def get_doc(self):
        print("help pressed.")
        home_doc = resources.package_dirname("orangecontrib.oasysaddontemplate") + "/doc_files/"
        filename1 = os.path.join(home_doc,'{widget_class_name}'+'.txt')
        print("Opening file %s"%filename1)
        if sys.platform == 'darwin':
            command = "open -a TextEdit "+filename1+" &"
        elif sys.platform == 'linux':
            command = "gedit "+filename1+" &"
        os.system(command)


    #
    # this is the calculation method to be implemented by the user
    # It is defined as static method to get all inputs from the arguments so it
    # can easily moved outside the class
    #
    @staticmethod
    def calculate_external_{widget_class_name}({calc_args_default}):
        print("Inside calculate_external_{widget_class_name}. ")
        return(None)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = OW{widget_class_name}()
    w.show()
    app.exec()
    w.saveSettings()
"""

main()




#! /usr/bin/env python3

import imp
import os
import sys
from setuptools import find_packages, setup
import subprocess

NAME = 'OASYS1-COMSYL'
VERSION = '1.0.18'
ISRELEASED = False

DESCRIPTION = 'oasys-comsyl: Oasys widgets for COMSYL'
README_FILE = os.path.join(os.path.dirname(__file__), 'README.txt')
LONG_DESCRIPTION = open(README_FILE).read()
AUTHOR = 'M. Sanchez del Rio, M. Glass'
AUTHOR_EMAIL = 'srio@esrf.eu'
URL = 'https://github.com/oasys-kit/oasys-comsyl'
DOWNLOAD_URL = 'https://github.com/oasys-kit/oasys-comsyl'
LICENSE = 'MIT'

KEYWORDS = (
    'application',
    'COMSYL',
    'Oasys',
    'Orange',
    'comsyl>=1.0.13',
    )

CLASSIFIERS = (
    'Development Status :: 4 - Beta',
    'Environment :: X11 Applications :: Qt',
    'Environment :: Console',
    'Environment :: Plugins',
    'Programming Language :: Cython',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Intended Audience :: Science/Research',
    )


SETUP_REQUIRES = (
                  'setuptools',
                  )

INSTALL_REQUIRES = (
                    'oasys1',
                    'comsyl'
                    )

PACKAGES = find_packages(exclude=('*.tests', '*.tests.*', 'tests.*', 'tests'))



PACKAGE_DATA = {
    "":["icons/*.png", "icons/*.jpg"],
    # "orangecontrib.comsyl.widgets.applications":["icons/*.png", "icons/*.jpg"],
    # "orangecontrib.comsyl.widgets.tools":["icons/*.png", "icons/*.jpg"],
    }

NAMESPACE_PACAKGES = ["orangecontrib","orangecontrib.comsyl", "orangecontrib.comsyl.widgets"]


ENTRY_POINTS = {
    'oasys.addons' : ("COMSYL = orangecontrib.comsyl", ),
    'oasys.widgets' : (
            "COMSYL Apps = orangecontrib.comsyl.widgets.applications",
            "COMSYL Tools = orangecontrib.comsyl.widgets.tools",
            ),
    }

if __name__ == '__main__':
    setup(
          name = NAME,
          version = VERSION,
          description = DESCRIPTION,
          long_description = LONG_DESCRIPTION,
          author = AUTHOR,
          author_email = AUTHOR_EMAIL,
          url = URL,
          download_url = DOWNLOAD_URL,
          license = LICENSE,
          keywords = KEYWORDS,
          classifiers = CLASSIFIERS,
          packages = PACKAGES,
          package_data = PACKAGE_DATA,
          setup_requires = SETUP_REQUIRES,
          install_requires = INSTALL_REQUIRES,
          entry_points = ENTRY_POINTS,
          namespace_packages=NAMESPACE_PACAKGES,
          include_package_data = True,
          zip_safe = False,
          )



__author__ = 'srio'

from silx.gui.data.DataViewer import DataViewer

import numpy

from PyQt4 import QtGui
import h5py


app = QtGui.QApplication([])

hfile = "/Users/srio/OASYS_VE/oasys-comsyl/orangecontrib/comsyl/scripts/ph3_u18_3_17keV_s1.3_100modes.h5"

h5f = h5py.File(hfile,'r')
for key in h5f.keys():
    print(">>>>",key)
data = h5f['modes'][:]
h5f.close()

print(">>>>>>>>>>",data.shape)


viewer = DataViewer()
viewer.setData(numpy.absolute(data)**2)
viewer.setVisible(True)

app.exec_()




import numpy
from oasys.util.oasys_util import get_fwhm

from srxraylib.plot.gol import plot, plot_image
import matplotlib.pylab as plt
import os, sys, time
import h5py
from srxraylib.util.h5_simple_writer import H5SimpleWriter

# def get_fwhm(histogram, bins):
#     quote = numpy.max(histogram)*0.5
#     cursor = numpy.where(histogram >= quote)
#
#     if histogram[cursor].size > 1:
#         bin_size    = bins[1]-bins[0]
#         fwhm        = bin_size*(cursor[0][-1]-cursor[0][0])
#         coordinates = (bins[cursor[0][0]], bins[cursor[0][-1]])
#     else:
#         fwhm = 0.0
#         coordinates = None
#
#     return fwhm, quote, coordinates

#
#
#
class Tally2D():
    def __init__(self,
                 scan_variable_name='x',
                 additional_stored_variable_names=None,
                 do_store_wavefronts=False):
        self.reset()
        self.scan_variable_name = scan_variable_name
        self.additional_stored_variable_names = additional_stored_variable_names
        self.do_store_wavefronts = do_store_wavefronts

    def reset(self):
        self.scan_variable_index = -1
        self.scan_variable_value = []
        self.fwhm_x = []
        self.fwhm_y = []
        self.intensity_at_center = []
        self.intensity_total = []
        self.intensity_peak = []
        self.intensity_accumulated = None
        self.coordinate_x = None
        self.coordinate_y = None
        self.additional_stored_values = []
        self.stored_wavefronts = []


    def append(self, wf, scan_variable_value=None, additional_stored_values=None):
        fwhm_x, fwhm_y, intensity_total, intensity_at_center, intensity_peak, intensity, x, y = self.process_wavefront_2d(wf)
        self.fwhm_x.append(fwhm_x)
        self.fwhm_y.append(fwhm_y)
        self.intensity_at_center.append(intensity_at_center)
        self.intensity_total.append(intensity_total)
        self.intensity_peak.append(intensity_peak)
        self.scan_variable_index += 1
        if scan_variable_value is None:
            self.scan_variable_value.append(self.scan_variable_index)
        else:
            self.scan_variable_value.append(scan_variable_value)

        self.additional_stored_values.append(additional_stored_values)

        if self.do_store_wavefronts:
            self.stored_wavefronts.append(wf.duplicate())

        if self.intensity_accumulated is None:
            self.intensity_accumulated = intensity
        else:
            self.intensity_accumulated += intensity

        self.coordinate_x = x
        self.coordinate_y = y

    def get_wavefronts(self):
        return self.stored_wavefronts

    def get_number_of_calls(self):
        return self.scan_variable_index + 1

    def get_additional_stored_values(self):
        return self.additional_stored_values

    def save_scan(self, filename="tmp.dat", add_header=True):
        raise Exception("Te be implemented")
        # f = open(filename, 'w')
        # if add_header:
        #     if self.additional_stored_variable_names is None:
        #         number_of_additional_parameters = 0
        #     else:
        #         number_of_additional_parameters = len(self.additional_stored_variable_names)
        #     header = "#S 1 scored data\n"
        #     header += "#N %d\n" % (number_of_additional_parameters + 5)
        #     header_titles = "#L  %s  %s  %s  %s  %s" % (self.scan_variable_name, "fwhm", "total_intensity", "on_axis_intensity", "peak_intensity")
        #     for i in range(number_of_additional_parameters):
        #         header_titles += "  %s" % self.additional_stored_variable_names[i]
        #     header_titles += "\n"
        #     header += header_titles
        #     f.write(header)
        # for i in range(len(self.fwhm)):
        #     f.write("%g %g %g %g %g" % (self.scan_variable_value[i],
        #                             1e6*self.fwhm[i],
        #                             self.intensity_total[i],
        #                             self.intensity_at_center[i],
        #                             self.intensity_peak[i]))
        #     for j in range(number_of_additional_parameters):
        #         f.write(" %g" % self.additional_stored_values[i][j])
        #     f.write("\n")
        # f.close()
        # print("File written to disk: %s" % filename)

    def get_fwhm_intensity_accumulated(self):

        nx = self.coordinate_x.size
        ny = self.coordinate_y.size

        fwhm_x, quote, coordinates = get_fwhm(self.intensity_accumulated[:, ny//2], self.coordinate_x)
        fwhm_y, quote, coordinates = get_fwhm(self.intensity_accumulated[nx // 2, :], self.coordinate_y)

        return fwhm_x, fwhm_y

    def get_histograms(self):
        hx = self.intensity_accumulated.sum(axis=1)
        hy = self.intensity_accumulated.sum(axis=0)
        return hx, hy

    def get_fwhm_histograms(self):
        hx, hy = self.get_histograms()
        fwhm_x, quote, coordinates = get_fwhm(hx, self.coordinate_x)
        fwhm_y, quote, coordinates = get_fwhm(hy, self.coordinate_y)
        return fwhm_x, fwhm_y

    def plot_intensity_accumulated(self, show=1, filename="", aspect=None,
                                   title="intensity accumulated", xtitle="x", ytitle="y",
                                   coordinates_factor=1.0):

        fwhm_x, fwhm_y = self.get_fwhm_intensity_accumulated()

        plot_image(self.intensity_accumulated,
                   coordinates_factor * self.coordinate_x,
                   coordinates_factor * self.coordinate_y,
                   title="%s fwhm of central profiles=%g x %g " % (title, coordinates_factor*fwhm_x, coordinates_factor * fwhm_y),
                   xtitle=xtitle, ytitle=ytitle, aspect=aspect, show=False)

        if filename != "":
            plt.savefig(filename)
            print("File written to disk: %s" % filename)

        if show:
            plt.show()
        else:
            plt.close()

    def save(self, filename=""):

        if filename == "": return

        fwhm_profile_x, fwhm_profile_y = self.get_fwhm_intensity_accumulated()
        fwhm_histo_x, fwhm_histo_y = self.get_fwhm_histograms()

        Z = self.intensity_accumulated
        hx, hy = self.get_histograms()
        x_coordinates = 1e3 * self.coordinate_x
        y_coordinates = 1e3 * self.coordinate_y

        #
        # initialize file
        #
        h5w = H5SimpleWriter.initialize_file(filename, creator="h5_basic_writer.py")

        # this is optional
        h5w.set_label_image("intensity_accumulated", b'x', b'y')
        h5w.set_label_dataset(b'abscissas', b'intensity')


        # create the entry for this iteration and set default plot to "Wintensity"
        h5w.create_entry("accumulated_intensity", nx_default="intensity")


        # add the images at this entry level
        h5w.add_image(Z, 1e3 * x_coordinates, 1e3 * y_coordinates,
                      entry_name="accumulated_intensity", image_name="intensity",
                      title_x="X [um]", title_y="Y [um]")


        h5w.add_dataset(1e3 * x_coordinates, Z[:, int(y_coordinates.size / 2)],
                        entry_name="accumulated_intensity", dataset_name="profileH",
                        title_x="X [um]", title_y="Profile along X")
        h5w.add_key("fwhm", 1e6*fwhm_profile_x, entry_name="accumulated_intensity/profileH")


        h5w.add_dataset(1e3 * y_coordinates, Z[int(x_coordinates.size / 2), :],
                        entry_name="accumulated_intensity", dataset_name="profileV",
                        title_x="Y [um]", title_y="Profile along Y")
        h5w.add_key("fwhm", 1e6 * fwhm_profile_y, entry_name="accumulated_intensity/profileV")


        h5w.add_dataset(1e3 * x_coordinates, hx,
                        entry_name="accumulated_intensity", dataset_name="histogramH",
                        title_x="X [um]", title_y="Histogram along X")
        h5w.add_key("fwhm", 1e6 * fwhm_histo_x, entry_name="accumulated_intensity/histogramH")


        h5w.add_dataset(1e3 * y_coordinates, hy,
                        entry_name="accumulated_intensity", dataset_name="histogramV",
                        title_x="Y [um]", title_y="Histogram along Y")
        h5w.add_key("fwhm", 1e6 * fwhm_histo_y, entry_name="accumulated_intensity/histogramV")

        print("File written to disk: %s" % filename)

    @classmethod
    def process_wavefront_2d(cls, wf):
        I = wf.get_intensity()
        x = wf.get_coordinate_x()
        y = wf.get_coordinate_y()

        nx = x.size
        ny = y.size

        fwhm_x, quote, coordinates = get_fwhm(I[:, ny//2], x)
        fwhm_y, quote, coordinates = get_fwhm(I[nx//2, :], y)

        intensity_at_center = I[nx//2, ny//2]
        intensity_total = I.sum() * (x[1] - x[0]) * (y[1] - y[0])
        intensity_peak = I.max()

        return fwhm_x, fwhm_y, intensity_total, intensity_at_center, intensity_peak, I, x, y



if __name__ == "__main__":
    from wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D

    sc = Tally2D(scan_variable_name='mode index', additional_stored_variable_names=['a', 'b'], do_store_wavefronts=False)
    for xmode in range(5):
        output_wavefront = GenericWavefront2D.initialize_wavefront_from_range(x_min=-0.00012, x_max=0.00012,
                                                                              y_min=-5e-05, y_max=5e-05,
                                                                              number_of_points=(400, 200))
        output_wavefront.set_photon_energy(7000)
        output_wavefront.set_gaussian_hermite_mode(sigma_x=3.00818e-05, sigma_y=6.99408e-06, amplitude=1, nx=xmode, ny=0,
                                                   betax=0.129748, betay=1.01172)


        sc.append(output_wavefront, scan_variable_value=xmode, additional_stored_values=[1,2.1])

    print(sc.fwhm_x, sc.fwhm_y)
    # plot_image(sc.intensity_accumulated, sc.coordinate_x, sc.coordinate_y)

    sc.plot_intensity_accumulated(coordinates_factor=1e6, aspect='auto')
    # sc.plot()
    sc.save("tmp.h5")




import os
import numpy

from PyQt5.QtWidgets import QMessageBox

from orangewidget import gui, widget
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui, congruence


from comsyl.autocorrelation.CompactAFReader import CompactAFReader
from wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D


class OWModesFileWriter(widget.OWWidget):
    name = "Modes to File"
    description = "Utility: COMSYL Modes File Writer"
    icon = "icons/file_writer.png"
    maintainer = "Manuel Sanchez del Rio"
    maintainer_email = "srio(@at@)esrf.eu"
    priority = 61
    category = "Utility"
    keywords = ["COMSYL", "coherent modes"]

    want_main_area = 0

    file_name = Setting("tmp.h5")
    index_format = Setting("%04d")
    is_automatic_run= Setting(1)
    TYPE_OF_OUTPUT = Setting(1) # 'COMSYL hdf5 with multi-mode','WOFRY hdf5 with multi-mode','WOFRY multiple files'
    ALL_MODES = Setting(0)
    MODE_TO = Setting(10)

    inputs = [("COMSYL modes" , CompactAFReader, "setCompactAFReader" )]


    af = None

    def __init__(self):
        super().__init__()

        self.runaction = widget.OWAction("Write HDF5 File", self)
        self.runaction.triggered.connect(self.write_file)
        self.addAction(self.runaction)

        self.setFixedWidth(590)
        self.setFixedHeight(500)

        left_box_1 = oasysgui.widgetBox(self.controlArea, "HDF5 File Selection", addSpace=True, orientation="vertical",
                                         width=570, height=400)

        gui.checkBox(left_box_1, self, 'is_automatic_run', 'Automatic Execution')

        gui.separator(left_box_1, height=10)

        figure_box = oasysgui.widgetBox(left_box_1, "", addSpace=True, orientation="horizontal", width=550, height=50)


        #
        #
        #
        self.le_file_name = oasysgui.lineEdit(figure_box, self, "file_name", "Output File Name",
                                                    labelWidth=120, valueType=str, orientation="horizontal")
        self.le_file_name.setFixedWidth(330)


        gui.button(figure_box, self, "...", callback=self.selectFile)

        gui.separator(left_box_1, height=10)


        #
        #
        #

        gui.comboBox(left_box_1, self, "TYPE_OF_OUTPUT", label=" Type of output file", labelWidth=260,
                     items=['COMSYL hdf5 with multi-mode','WOFRY hdf5 with multi-mode','WOFRY multiple files'],
                     #callback=self.set_Propagator,
                     sendSelectedValue=False, orientation="horizontal")

        self.le_index_format = oasysgui.lineEdit(left_box_1, self, "index_format", "index format [for individual files]",
                                                    labelWidth=200, valueType=str, orientation="horizontal")
        gui.separator(left_box_1, height=10)



        #
        #
        #
        gui.comboBox(left_box_1, self, "ALL_MODES", label=" Modes to write", labelWidth=260,
                     items=['Selected modes','All modes'],
                     #callback=self.set_Propagator,
                     sendSelectedValue=False, orientation="horizontal")


        oasysgui.lineEdit(left_box_1, self, "MODE_TO", "To mode index:",
                                                    labelWidth=200, valueType=int, orientation="horizontal")

        gui.separator(left_box_1, height=10)
        #
        #
        #
        button = gui.button(self.controlArea, self, "Write File", callback=self.write_file)
        button.setFixedHeight(45)
        self.le_index_format.setFixedWidth(330)

        gui.rubber(self.controlArea)

    def selectFile(self):
        self.le_file_name.setText(oasysgui.selectFileFromDialog(self, self.file_name, "Open HDF5 File"))

    def setCompactAFReader(self, data):
        if not data is None:
            self.af = data

            if self.is_automatic_run:
                self.write_file()


    def write_file(self):
        self.setStatusMessage("")

        try:
            if not self.af is None:
                congruence.checkDir(self.file_name)


                if self.TYPE_OF_OUTPUT == 0: # ['COMSYL hdf5 with multi-mode','WOFRY hdf5 with multi-mode','WOFRY multiple files'
                    if self.ALL_MODES:
                        self.af.write_h5(self.file_name,maximum_number_of_modes=None)
                    else:
                        self.af.write_h5(self.file_name,maximum_number_of_modes=self.MODE_TO)
                        path, file_name = os.path.split(self.file_name)
                        self.setStatusMessage("File Out: " + file_name)
                else: # WOFRY
                    if self.ALL_MODES == 0 and (self.MODE_TO < self.af.number_of_modes()):
                        nmax = self.MODE_TO
                    else:
                        nmax = self.af.number_of_modes()

                    for i in range(nmax+1):
                        eigenvalue = numpy.real(self.af.eigenvalue(i),)
                        eigenfunction = self.af.mode(i)
                        w = GenericWavefront2D.initialize_wavefront_from_arrays(
                            self.af.x_coordinates(), self.af.y_coordinates(),
                            eigenfunction*numpy.sqrt(eigenvalue))
                        w.set_photon_energy(self.af.photon_energy() )


                        if self.TYPE_OF_OUTPUT == 1: #
                            file_name = self.file_name
                            subgroupname = "mode"+self.index_format%(i)
                            overwrite = False
                        elif self.TYPE_OF_OUTPUT == 2:
                            subgroupname = "mode"+self.index_format%(i)
                            file_name = self.file_name.split(".")[0]+"_"+(self.index_format%i)+".h5"
                            overwrite = True

                        if i==0:
                            w.save_h5_file(file_name,subgroupname=subgroupname,intensity=True,phase=True,overwrite=True)
                        else:
                            w.save_h5_file(file_name,subgroupname=subgroupname,intensity=True,phase=True,overwrite=overwrite)
                        path, file_name = os.path.split(file_name)
                        self.setStatusMessage("File Out: " + file_name)



                # path, file_name = os.path.split(self.file_name)

                # self.setStatusMessage("File Out: " + file_name)

            else:
                QMessageBox.critical(self, "Error",
                                     "COMSYL modes not present",
                                     QMessageBox.Ok)
        except Exception as exception:
            QMessageBox.critical(self, "Error", str(exception), QMessageBox.Ok)


if __name__ == "__main__":
    from PyQt5.QtWidgets import QApplication
    import sys

    a = QApplication(sys.argv)
    ow = OWModesFileWriter()
    ow.file_name = "tmp.h5"

    filename = "/users/srio/COMSYLD/comsyl/comsyl/calculations/septest_cm_new_u18_2m_1h_s2.5.npy"
    af = CompactAFReader.initialize_from_file(filename)
    ow.af = af

    ow.show()
    a.exec_()